<?php

namespace Teknisa\CADCTB\Services;

use Zeedhi\Framework\DTO\Response;
use Teknisa\Libs\Util\DateUtil;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Ctb94000AtualizacaoCustoProducaoService{

    protected $environment;
    protected $nrorg;
    
    
    public function __construct(EntityManager $entityManager, Environment $environment) {
        $this->environment = $environment;
        $this->entityManager = $entityManager;
        $this->nrorg = $environment->getNrOrgTrab();
        $this->connection = $entityManager->getConnection();
    }

    
    public function verificaHistorico(){
        try{
            // Query a ser executada.
            $sql = "SELECT CDHISTPADRGGF, CDHISTPADRTRANSGGF FROM PARAMESTOQUE";
    
            // Prepara o statement da query para execução.
            $statement = $this->connection->prepare($sql);
    
            // Executa o statement com os paràmetros passados.
            $statement->execute();

            // Executa um fetch em todos os dados, retornando-os.
            $paramEstoque = $statement->fetchAll()[0];
            if($paramEstoque['CDHISTPADRGGF'] == 'null'){
                throw new \Exception("Não há Histórico padrão cadastrado para o Rateio GGF.",1);
            }else if($paramEstoque['CDHISTPADRTRANSGGF']== 'null'){
                throw new \Exception("Não há Histórico padrão cadastrado para o Transferência do GGF.",1);
            }
            else {
                return true;
            }
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function verificaPeriodo($params){
        try{
            // Query a ser executada.
            $sql = "SELECT C.CDEMPRESA, C.DTINICLANC, C.DTFINALANC
                        FROM
                         CONTOPECTBL C JOIN FILIAL FI ON C.CDEMPRESA = FI.CDEMPRESA
                        WHERE DTINICLANC <= :DTLANC
                         AND DTFINALANC >= :DTLANC
                         AND (FI.CDFILIAL = :CDFILIAL)";
    
            // Prepara o statement da query para execução.
            $statement = $this->connection->prepare($sql);
    
            // Executa o statement com os paràmetros passados.
            $statement->execute($params);

            // Executa um fetch em todos os dados, retornando-os.
            $periodoContabil = $statement->fetchAll();
            if(empty($periodoContabil)){
                throw new \Exception("Data do lançamento fora do período contábil.",1);
            }else{
                return true;
            }
            
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
        
    public function execAtualizaCustoAbsorcao($row){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'ATUALIZA_CUSTO_ABSORCAO');
        $procedure->addParam(new Param('P_CDFILIAL',  Param::PARAM_INPUT, $row['CDFILIAL'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATACUSTO', Param::PARAM_INPUT, '01/'.$row['DTLANC'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODINI', Param::PARAM_INPUT, $row['CDPRODINI'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDPRODFIN', Param::PARAM_INPUT, $row['CDPRODFIN'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ERRO', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 1));
        $output = $procedure->execute();

        // Verifica se o o parametro de erro retornou, e se não manda erro.
        $P_ERRO = !empty($output['P_ERRO']) ? $output['P_ERRO'] : 'S';
        
        if($P_ERRO== 'S'){
          throw new \Exception("Operação não realizada! Não há saldo contábil ou custo de absorção já calculado para o período.",1);
        }else{
            return true;
        }
    
    }
    
    public function execCancelaCustoAbsorcao($row){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CANCELA_CUSTO_ABSORCAO');
        $procedure->addParam(new Param('P_CDFILIAL',  Param::PARAM_INPUT, $row['CDFILIAL'],  Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATACUSTO', Param::PARAM_INPUT, '01/'.$row['DTLANC'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ERRO', Param::PARAM_OUTPUT, null, Param::PARAM_TYPE_STR, 1));
        $output = $procedure->execute();
        
        // Verifica se o o parametro de erro retornou, e se não manda erro.
        $P_ERRO = !empty($output['P_ERRO']) ? $output['P_ERRO'] : 'S';
        
        if($P_ERRO== 'S'){
          throw new \Exception("Operação não realizada! Custo de absorção não calculado para o mês informado.",1);
        }else{
            return true;
        }
        
    }
    
}