/* TEKNISA

Nome: 7601_PMESTPO_CONSUMO_PLANEJADO_PME.SQL

Descrio:
Clculo do Consumo Planejado (Cardpio, Consumo por Servio Fixo ou
Percapita) e gravao na tabela CONSPLAN ou PREVRETI.

P_FILIAL       - Codigo da Filial
P_SERVICOS     - String contendo sequencia de Servios a serem calculados
P_DTINICIAL    - Data inicial
P_DTFINAL      - Data final
P_IDARQUIVO    - Identificador da tabela a ser gravada
                 "P" -> PREVRETI
                 "C" -> CONSPLAN
                 "A" -> AMBAS
P_IDPRODUTOS   - Identificador dos produtos a serem gravados.
                 Formato "XXX"
                 1 posio = "C" -> Cardpio
                 2 posio = "F" -> Consumo por Servio Fixo
                 3 posio = "P" -> Consumo por Servio Percapita
P_IDSEPPRATO   - Identificador de requisio analtica, ou seja, separada
                 por prato ou por setor, consumo fixo e consumo percapita.
                 Formato "S" (setor), "P" (prato) ou "N" (n/a).
P_PEACRESCIMO  - Percentual de acrscimo
P_IDARREDOND   - Identificador de utilizao do arredondamento
                 para requisio do Produto. Formato
                    S - ARREDONDAMENTO DE REQUISIO
                    N - NAO ARREDONDAR
                    C - ARREDONDAMENTO DE COMPRAS

P_NRCONSPLAN   - Nmero sequencial a ser utilizado para gravao dos dados
P_CDPRODUTOINI - Cdigo do Produto inicial
P_CDPRODUTOFIN - Cdigo do Produto final
P_CDOPERADOR   - Cdigo do Operador
P_STDTINICIAL  - Lista de data inicial em string por servio
P_STDTFINAL    - Lista de data final em string por servio

Para aplicar e debugar
-- @C:\Web_409\Sql\End_Conv\Oracle\7601_PMESTPO_CONSUMO_PLANEJADO_PME.SQL
-- EXEC CONSUMO_PLANEJADO_PME('0751', NULL,'0763','00030','25/09/2006',
                              '25/09/2006','A','CFP','P',0,'N','0000001127',
                              '0','zzzzzzzzzz','000000099999')

Criao:
20/05/2005 - Luiz Otvio de Souza

*/

CREATE OR REPLACE PROCEDURE CONSUMO_PLANEJADO_PME
  (P_CDFILCENTPME IN VARCHAR2,
   P_CDTIPOCARD   IN VARCHAR2,
   P_CDFILIAL     IN VARCHAR2,
   P_CDSERVFILI   IN VARCHAR2,
   P_DTINICIAL    IN DATE,
   P_DTFINAL      IN DATE,
   P_IDARQUIVO    IN VARCHAR2,
   P_IDPRODUTOS   IN VARCHAR2,
   P_IDSEPPRATO   IN VARCHAR2,
   P_PEACRESCIMO  IN NUMBER,
   P_IDARREDOND   IN VARCHAR2,
   P_NRCONSPLAN   IN VARCHAR2,
   P_CDPRODUTOINI IN VARCHAR2,
   P_CDPRODUTOFIN IN VARCHAR2,
   P_CDOPERADOR   IN VARCHAR2)
IS
   V_COUNTSERVICO  NUMBER;
   V_ARRED         NUMBER;
   V_IDCARDAPIO    VARCHAR2(1);
   V_IDCONSFIXO    VARCHAR2(1);
   V_IDCONSPERC    VARCHAR2(1);
   V_DTLIMITE      DATE;
   V_HRLIMITE      VARCHAR2(4);
   V_CDFAPRATCPLA  CONSPLAN.CDFAPRATCPLA%TYPE;
   V_CDPRATCONSPL  CONSPLAN.CDPRATCONSPL%TYPE;
   V_CDSETOCONSPL  CONSPLAN.CDSETOCONSPL%TYPE;
   V_CDTPPRATCPLA  CONSPLAN.CDTPPRATCPLA%TYPE;
   V_IDUTILFATCOR  PARAMPLANEJA.IDUTILFATCOR%TYPE;
   V_IDUTILFATCOC  PARAMPLANEJA.IDUTILFATCOC%TYPE;
   V_IDTIPCALCFAT  PARAMPLANEJA.IDTIPCALCFAT%TYPE;
   V_IDUTREQUPRAT  PARAMPLANEJA.IDUTREQUPRAT%TYPE;
   V_IDUTRETIPRAT  SERVICO.IDUTRETIPRAT%TYPE;
   V_CDSETCONFIXO  PARAMPLANEJA.CDSETCONFIXO%TYPE;
   V_CDSETCONPERC  PARAMPLANEJA.CDSETCONPERC%TYPE;
   V_QTCOMESTICAR  CARDADIA.QTCOMESTICAR%TYPE;
   V_QTESTITURNO1  CARDADIA.QTESTITURNO1%TYPE;
   V_QTESTITURNO2  CARDADIA.QTESTITURNO2%TYPE;
   V_QTESTITURNO3  CARDADIA.QTESTITURNO3%TYPE;
   V_QTESTITURNO4  CARDADIA.QTESTITURNO4%TYPE;
   V_QTDEAUX       NUMBER;
   V_QTPRATO       NUMBER;
   V_FERIADO       NUMBER;
   V_CONTTURNO     NUMBER;
   V_CONTTURNCARD  NUMBER;
   V_QTPREV        NUMBER;
   V_QTPREV1       NUMBER;
   V_QTPREV2       NUMBER;
   V_QTPREV3       NUMBER;
   V_QTPREV4       NUMBER;
   V_CDTPRCONSFIX  CONSPLAN.CDTPPRATCPLA%TYPE;
   V_CDFILCONSFIX  CONSPLAN.CDFAPRATCPLA%TYPE;
   V_CDPRACONSFIX  CONSPLAN.CDPRATCONSPL%TYPE;
   V_CDTPRCONSPER  CONSPLAN.CDTPPRATCPLA%TYPE;
   V_CDFILCONSPER  CONSPLAN.CDFAPRATCPLA%TYPE;
   V_CDPRACONSPER  CONSPLAN.CDPRATCONSPL%TYPE;
   V_NRPRPREVRETI  VARCHAR2(4);
   V_SERV_ATUAL    VARCHAR2(5);
   V_CDTCACONSPER  VARCHAR2(10);
   V_CDTCACONSFIX  VARCHAR2(10);
   V_TPCARDCONSFIX VARCHAR2(10);
   V_IDPRODNAREQPREV VARCHAR2(1);
   V_PERCACRES     NUMBER;
   CUR_PRODPRAT    UTIL_CONSPLAN_PME.PRODPRAT_PME_CURTYPE;
   REC_PRODPRAT    UTIL_CONSPLAN_PME.PRODPRAT_PME_RECTYPE;
   CUR_ARREDOND    UTIL_CONSPLAN_PME.ARREDOND_PME_CURTYPE;
   REC_ARREDOND    UTIL_CONSPLAN_PME.ARREDOND_PME_RECTYPE;
   CUR_CONSSERVPME UTIL_CONSPLAN_PME.CONSSERVPME_CURTYPE;
   REC_CONSSERVPME UTIL_CONSPLAN_PME.CONSSERVPME_RECTYPE;
   CUR_CONSPLAN    UTIL_CONSPLAN_PME.CONSPLAN_PME_CURTYPE;
   REC_CONSPLAN    UTIL_CONSPLAN_PME.CONSPLAN_PME_RECTYPE;
   /* constantes */
   w_S CHAR(1) := 'S';

BEGIN
   /* --------------- IDENTIFICADORES DE CLCULO ------------------- */
   V_IDCARDAPIO := SUBSTR(P_IDPRODUTOS, 1, 1);  /* CARDPIO          */
   V_IDCONSFIXO := SUBSTR(P_IDPRODUTOS, 2, 1);  /* CONSUMO FIXO      */
   V_IDCONSPERC := SUBSTR(P_IDPRODUTOS, 3, 1);  /* CONSUMO PERCAPITA */

   /* -- CALCULO DOS PRODUTOS DOS CARDAPIOS -- */
   IF (V_IDCARDAPIO = 'C') THEN
      SELECT
         MAX(IDUTILFATCOR),
         MAX(IDUTILFATCOC),
         MAX(IDTIPCALCFAT)
      INTO
         V_IDUTILFATCOR,
         V_IDUTILFATCOC,
         V_IDTIPCALCFAT
      FROM
         PARAMPLANEJA
      WHERE
         CDFILIAL = P_CDFILCENTPME;

      IF V_IDUTILFATCOC IS NULL THEN
         V_IDUTILFATCOC := 'S';
      END IF;
      IF V_IDUTILFATCOR IS NULL THEN
         V_IDUTILFATCOR := 'S';
      END IF;
      IF V_IDTIPCALCFAT IS NULL THEN
         V_IDTIPCALCFAT := 'T';
      END IF;
      IF V_IDUTREQUPRAT IS NULL THEN
         V_IDUTREQUPRAT := 'N';
      END IF;

      FOR C_CARDAPIO IN (
         SELECT CD.CDFILCENTPME, CD.NRCARDPME,   CD.NRCARDIAPME,  CD.CDSERVFILI,
                CD.CDFILIAL,     CD.DTCARDIAPME, CD.QTCOMESTCPME, CD.IDCARDESPEC,
                C.CDTIPOCARD
         FROM CARDAPIOPME C, CARDIAPME CD, FILIAL F, FILITIPOCARD TC
         WHERE (C.CDFILCENTPME = P_CDFILCENTPME)                            AND
               (INSTR(NVL(P_CDTIPOCARD, C.CDTIPOCARD), C.CDTIPOCARD) > 0)   AND
               (INSTR(NVL(P_CDFILIAL, CD.CDFILIAL), CD.CDFILIAL) > 0)       AND
               (C.CDFILCENTPME = CD.CDFILCENTPME)                           AND
               (C.NRCARDPME    = CD.NRCARDPME)                              AND
               (CD.CDFILIAL    <> CD.CDFILCENTPME)                          AND
               (INSTR(NVL(P_CDSERVFILI, CD.CDSERVFILI), CD.CDSERVFILI) > 0) AND
               (F.CDFILCENTPME = C.CDFILCENTPME)                            AND
               (F.CDFILIAL     = CD.CDFILIAL   )                            AND
               (CD.DTCARDIAPME BETWEEN P_DTINICIAL AND P_DTFINAL)           AND
               (TC.CDFILCENTPME = C.CDFILCENTPME)                           AND
               (TC.CDTIPOCARD   = C.CDTIPOCARD)                             AND
               (TC.CDFILIAL     = CD.CDFILIAL)                              AND
               (TC.IDTCARATFILI = 'S')
         ORDER BY CD.CDFILIAL, CD.CDSERVFILI, CD.DTCARDIAPME)
      LOOP
         /* ---- LOOP PARA CALCULO DOS CARDAPIOS ---- */
         IF (C_CARDAPIO.QTCOMESTCPME <> 0) THEN  /* NO CALCULA PRATOS PARA COMENSAIS IGUAL A ZERO */
            FOR C_COMPCARD IN (
               SELECT
                  CC.CDPRATO, CC.CDFILAUXPRAT,
                  SUM(CASE WHEN (CD.TIPO = 'P') AND (PP.VRPERACEIT IS NOT NULL) AND (PP.VRPERACEIT <> 0) THEN
                        100 * (1 - (PP.VRPERACEIT/100) * -1)
                     ELSE CC.VRPECONSCPME END) AS VRPECONSCPME
               FROM COMPCARDPME CC
                    LEFT JOIN PERACEITPRATO PP ON PP.CDFILCENTPME = C_CARDAPIO.CDFILCENTPME AND
                                    PP.CDFILIAL = C_CARDAPIO.CDFILIAL AND
                                    PP.CDPRATO = CC.CDPRATO AND
                                    PP.CDFILAUXPRAT = CC.CDFILAUXPRAT AND
                                    PP.CDTIPOCARD = C_CARDAPIO.CDTIPOCARD, CARDIAPME CA,
                    (SELECT 'P' AS TIPO, CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDSERVFILI, CDFILIAL, DTCARDIAPME
                     FROM CARDIAPME P
                     WHERE (P.CDFILCENTPME = C_CARDAPIO.CDFILCENTPME) AND
                           (P.NRCARDPME    = C_CARDAPIO.NRCARDPME)    AND
                           (P.CDSERVFILI   = C_CARDAPIO.CDSERVFILI)   AND
                           (P.CDFILIAL     = P.CDFILCENTPME)          AND
                           (P.DTCARDIAPME  = C_CARDAPIO.DTCARDIAPME)  AND
                           NOT EXISTS(SELECT CDFILIAL
                                      FROM  CARDIAPME C
                                      WHERE (C.CDFILCENTPME = P.CDFILCENTPME)      AND
                                            (C.NRCARDPME    = P.NRCARDPME)         AND
                                            (C.CDSERVFILI   = P.CDSERVFILI)        AND
                                            (C.CDFILIAL     = C_CARDAPIO.CDFILIAL) AND
                                            (C.DTCARDIAPME  = P.DTCARDIAPME)       AND
                                            (C.IDCARDESPEC  = w_S) )
                     UNION ALL
                     SELECT 'E' AS TIPO, CDFILCENTPME, NRCARDPME, NRCARDIAPME, CDSERVFILI, CDFILIAL, DTCARDIAPME
                     FROM CARDIAPME
                     WHERE (CDFILCENTPME = C_CARDAPIO.CDFILCENTPME) AND
                           (NRCARDPME    = C_CARDAPIO.NRCARDPME)    AND
                           (CDSERVFILI   = C_CARDAPIO.CDSERVFILI)   AND
                           (CDFILIAL     = C_CARDAPIO.CDFILIAL)     AND
                           (IDCARDESPEC  = w_S)                     AND
                           (DTCARDIAPME  = C_CARDAPIO.DTCARDIAPME)) CD
               WHERE (CC.CDFILCENTPME = CD.CDFILCENTPME) AND
                     (CC.NRCARDPME    = CD.NRCARDPME)    AND
                     (CC.NRCARDIAPME  = CD.NRCARDIAPME)  AND
                     (CA.CDFILCENTPME = CD.CDFILCENTPME) AND
                     (CA.NRCARDPME    = CD.NRCARDPME)    AND
                     (CA.CDSERVFILI   = CD.CDSERVFILI)   AND
                     (CA.DTCARDIAPME  = CD.DTCARDIAPME)  AND
                     (CA.CDFILIAL     = C_CARDAPIO.CDFILIAL)
               GROUP BY CC.CDPRATO, CC.CDFILAUXPRAT)
            LOOP
               IF (C_COMPCARD.VRPECONSCPME <> 0) THEN
                  V_QTPRATO := (C_COMPCARD.VRPECONSCPME * C_CARDAPIO.QTCOMESTCPME)/ 100 ;
                  UTIL_CONSPLAN_PME.OPEN_PRODPRAT_CURSOR_PME(CUR_PRODPRAT,
                                                             C_CARDAPIO.CDFILIAL,
                                                             C_CARDAPIO.CDSERVFILI,
                                                             C_COMPCARD.CDFILAUXPRAT,
                                                             C_COMPCARD.CDPRATO,
                                                             V_QTPRATO,
                                                             P_CDPRODUTOINI,
                                                             P_CDPRODUTOFIN,
                                                             V_IDUTILFATCOR,
                                                             V_IDUTILFATCOC,
                                                             V_IDTIPCALCFAT,
                                                             P_CDFILCENTPME,
                                                             C_CARDAPIO.CDTIPOCARD);
                  LOOP
                     FETCH CUR_PRODPRAT INTO REC_PRODPRAT;
                     EXIT WHEN CUR_PRODPRAT%NOTFOUND;

                     REC_PRODPRAT.QTPRODUTO := REC_PRODPRAT.QTPRODUTO * V_QTPRATO;
                     V_QTDEAUX := C_CARDAPIO.QTCOMESTCPME;

                     IF (V_QTDEAUX = 0) OR (V_QTDEAUX IS NULL) THEN
                        V_QTDEAUX := 1;
                     END IF;

                     V_CDFAPRATCPLA := ' ';                     
                     V_CDTPPRATCPLA := '  ';
                     
                     IF (P_IDSEPPRATO = 'P') THEN
                        V_CDFAPRATCPLA := C_COMPCARD.CDFILAUXPRAT;
                        V_CDPRATCONSPL := C_COMPCARD.CDPRATO;
                     ELSE
                        V_CDFAPRATCPLA := ' ';
                        V_CDPRATCONSPL := '          ';
                     END IF;
                     
                     BEGIN 
                        SELECT IDPRODNAREQPREV
                          INTO V_IDPRODNAREQPREV
                          FROM PRODFILI
                         WHERE CDFILIAL = C_CARDAPIO.CDFILIAL
                           AND CDPRODUTO = REC_PRODPRAT.CDPRODUTO;
                     EXCEPTION
                     WHEN NO_DATA_FOUND THEN V_IDPRODNAREQPREV := '';
                     END;
                   
                     IF (V_IDPRODNAREQPREV = 'S') THEN 
                        V_PERCACRES := 0;
                     ELSE 
                        V_PERCACRES := P_PEACRESCIMO;
                     END IF;

                     UTIL_CONSPLAN_PME.ATUALIZA_CONSPLAN_PME(P_NRCONSPLAN,
                                                             P_CDFILCENTPME,
                                                             C_CARDAPIO.CDFILIAL,
                                                             C_CARDAPIO.CDTIPOCARD,
                                                             C_CARDAPIO.CDSERVFILI,
                                                             'C',
                                                             V_CDFAPRATCPLA,
                                                             V_CDPRATCONSPL,
                                                             REC_PRODPRAT.CDAPRERECE,
                                                             V_CDSETOCONSPL,
                                                             REC_PRODPRAT.CDPRODUTO,
                                                             TO_DATE(TO_CHAR(C_CARDAPIO.DTCARDIAPME, 'DD/MM/YYYY'), 'DD/MM/YYYY'),
                                                             TO_CHAR(C_CARDAPIO.DTCARDIAPME, 'HH24MI'),
                                                             V_CDTPPRATCPLA,
                                                             (REC_PRODPRAT.QTPRODUTO) * (1 + (V_PERCACRES / 100)), 
                                                             C_COMPCARD.VRPECONSCPME,
                                                             V_QTPRATO,
                                                             'S',
                                                             C_CARDAPIO.DTCARDIAPME);
                  END LOOP; /* LOOP DA COMPOSICAO DOS PRATOS */
                  CLOSE CUR_PRODPRAT;
               END IF;
            END LOOP; /* LOOP DA COMPOSIO DO CARDAPIO */
         END IF; /* NO CALCULA PRATOS PARA COMENSAIS IGUAL A ZERO */
      END LOOP; /* LOOP PARA CALCULO DOS CARDAPIOS */
   END IF; /* CALCULO DOS PRODUTOS DOS CARDAPIOS */

   /* INICIO CONSUMO POR SERVICO */
   IF (V_IDCONSFIXO = 'F') OR (V_IDCONSPERC = 'P') THEN
      BEGIN
         SELECT
            CDTCACONSPER,
            CDTCACONSFIX
         INTO
            V_CDTCACONSPER,
            V_CDTCACONSFIX
         FROM
            PARAMPLANEJA
         WHERE
            (CDFILIAL = P_CDFILCENTPME);
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            NULL;
      END;

     IF (V_IDCONSPERC = 'P') AND (V_CDTCACONSPER IS NOT NULL) AND
        (V_IDCONSFIXO = 'F') AND (V_CDTCACONSFIX IS NOT NULL) THEN
         UTIL_CONSPLAN_PME.OPEN_CONSPLAN_CURSOR(CUR_CONSPLAN,
                                                P_CDFILCENTPME,
                                                P_CDFILIAL,
                                                P_DTINICIAL,
                                                P_DTFINAL,
                                                P_CDSERVFILI,
                                                'PF', /* BUSCAR CONSUMO FIXO E PER CAPITA*/
                                                P_NRCONSPLAN);
      ELSE
         IF (V_IDCONSPERC = 'P') AND (V_CDTCACONSPER IS NOT NULL) THEN
            UTIL_CONSPLAN_PME.OPEN_CONSPLAN_CURSOR(CUR_CONSPLAN,
                                                   P_CDFILCENTPME,
                                                   P_CDFILIAL,
                                                   P_DTINICIAL,
                                                   P_DTFINAL,
                                                   P_CDSERVFILI,
                                                   'P', /* BUSCAR PER CAPITA*/
                                                   P_NRCONSPLAN);
         ELSE
            IF (V_IDCONSFIXO = 'F') AND (V_CDTCACONSFIX IS NOT NULL) THEN
               UTIL_CONSPLAN_PME.OPEN_CONSPLAN_CURSOR(CUR_CONSPLAN,
                                                      P_CDFILCENTPME,
                                                      P_CDFILIAL,
                                                      P_DTINICIAL,
                                                      P_DTFINAL,
                                                      P_CDSERVFILI,
                                                      'F',  /* BUSCAR CONSUMO FIXO */
                                                      P_NRCONSPLAN);
            END IF;
         END IF;
      END IF;

      LOOP
         IF ((V_CDTCACONSPER IS NULL) AND (V_CDTCACONSFIX IS NULL)) THEN
            EXIT;
         END IF;

         FETCH CUR_CONSPLAN INTO REC_CONSPLAN;
         EXIT WHEN CUR_CONSPLAN%NOTFOUND;

         SELECT
            COUNT(*)
         INTO
            V_FERIADO
         FROM
            FERIFILI
         WHERE
            (CDFILIAL   = REC_CONSPLAN.CDFILIAL) AND
            (DTFERIFILI = REC_CONSPLAN.DTDATA);

         UTIL_CONSPLAN_PME.OPEN_CONSSERVPME_CURSOR(CUR_CONSSERVPME,
                                                   P_CDFILCENTPME,
                                                   REC_CONSPLAN.CDSERVFILI,
                                                   REC_CONSPLAN.CDFILIAL,
                                                   REC_CONSPLAN.DTDATA,
                                                   V_FERIADO,
                                                   P_CDPRODUTOINI,
                                                   P_CDPRODUTOFIN);
         LOOP
            FETCH CUR_CONSSERVPME INTO REC_CONSSERVPME;
            EXIT WHEN CUR_CONSSERVPME%NOTFOUND;

            IF ((V_IDCONSFIXO = 'F') AND (REC_CONSSERVPME.IDQTDECONPME = 'F')) OR
               ((V_IDCONSPERC = 'P') AND (NOT REC_CONSPLAN.NRCARDAPIO IS NULL)) THEN
               BEGIN
                  SELECT IDPRODNAREQPREV
                    INTO V_IDPRODNAREQPREV
                    FROM PRODFILI
                   WHERE CDFILIAL = REC_CONSPLAN.CDFILIAL
                     AND CDPRODUTO = REC_CONSSERVPME.CDPRODUTO;
               EXCEPTION
               WHEN NO_DATA_FOUND THEN V_IDPRODNAREQPREV := '';
               END;
                     
               IF (V_IDPRODNAREQPREV = 'S') THEN 
                  V_PERCACRES := 0;
               ELSE 
                  V_PERCACRES := P_PEACRESCIMO;
               END IF;               

               IF REC_CONSSERVPME.IDQTDECONPME = 'F' THEN
                  V_QTCOMESTICAR := (REC_CONSSERVPME.QTCONSERVPME) * (1 + (V_PERCACRES / 100)); 
               ELSE
                  V_QTCOMESTICAR := (REC_CONSSERVPME.QTCONSERVPME * REC_CONSPLAN.QTCOMESTICAR) * (1 + (V_PERCACRES / 100)); 
               END IF;

               IF (V_IDCONSFIXO = 'F' AND REC_CONSSERVPME.IDQTDECONPME = 'F') OR
                  (V_IDCONSPERC = 'P' AND REC_CONSSERVPME.IDQTDECONPME = 'P') THEN
                  IF P_IDARQUIVO <> 'P' THEN
                     V_CDFAPRATCPLA := ' ';
                     IF REC_CONSSERVPME.IDQTDECONPME = 'F' THEN
                        V_CDTPPRATCPLA := 'FF';
                     ELSE
                        V_CDTPPRATCPLA := 'PP';
                     END IF;

                     V_CDSETOCONSPL := '  ';

                     IF REC_CONSSERVPME.IDQTDECONPME = 'F' THEN
                        V_CDPRATCONSPL := 'FFFFFFFFFF';
                     ELSE
                        V_CDPRATCONSPL := 'PPPPPPPPPP';
                     END IF;
                     V_CDPRATCONSPL := '          ';
                     V_CDSETOCONSPL := '  ';

                     IF REC_CONSSERVPME.IDQTDECONPME = 'F' THEN
                        V_TPCARDCONSFIX := V_CDTCACONSFIX;
                     ELSE
                        V_TPCARDCONSFIX := V_CDTCACONSPER;
                     END IF;

                     UTIL_CONSPLAN_PME.ATUALIZA_CONSPLAN_PME(P_NRCONSPLAN,
                                                             P_CDFILCENTPME,
                                                             REC_CONSPLAN.CDFILIAL,
                                                             V_TPCARDCONSFIX,
                                                             REC_CONSPLAN.CDSERVFILI,
                                                             REC_CONSSERVPME.IDQTDECONPME,
                                                             V_CDFAPRATCPLA,
                                                             V_CDPRATCONSPL,
                                                             REC_PRODPRAT.CDAPRERECE,
                                                             V_CDSETOCONSPL,
                                                             REC_CONSSERVPME.CDPRODUTO,
                                                             REC_CONSPLAN.DTDATA,
                                                             REC_CONSPLAN.HRINICATEN,
                                                             V_CDTPPRATCPLA,
                                                             V_QTCOMESTICAR,
                                                             100,
                                                             REC_CONSPLAN.QTCOMESTICAR,
                                                             'N',
                                                             REC_CONSPLAN.DTDATA);
                  END IF;
               END IF;
            END IF;
         END LOOP;
         CLOSE CUR_CONSSERVPME;
      END LOOP;
      IF (NOT ((V_CDTCACONSPER IS NULL) AND (V_CDTCACONSFIX IS NULL))) THEN
         CLOSE CUR_CONSPLAN;
      END IF;
   END IF;
   /* FIM DO CONSUMO POR SERVICO */

   IF (P_IDARREDOND = 'S') OR (P_IDARREDOND = 'C') THEN
        UTIL_CONSPLAN_PME.OPEN_ARREDOND_CURSOR_PME(CUR_ARREDOND,
                                                   'C',
                                                   P_CDFILCENTPME,
                                                   P_CDSERVFILI,
                                                   P_DTINICIAL,
                                                   P_DTFINAL,
                                                   P_NRCONSPLAN,
                                                   P_NRCONSPLAN);
      LOOP
         FETCH CUR_ARREDOND INTO REC_ARREDOND;
         EXIT WHEN CUR_ARREDOND%NOTFOUND;
         IF (P_IDARREDOND = 'S') THEN
            V_ARRED := REC_ARREDOND.VRARREDREQ;
         ELSE
            V_ARRED := REC_ARREDOND.VRARREDCMP;
         END IF;
         IF NOT (V_ARRED IS NULL) AND (V_ARRED <> 0) THEN
            IF (REC_ARREDOND.QTCOMENST1 > 0) AND (MOD(REC_ARREDOND.QTCOMENST1, V_ARRED) > 0) THEN
               V_QTPREV1 := (FLOOR(REC_ARREDOND.QTCOMENST1/V_ARRED) + 1) * V_ARRED;
            ELSE
               V_QTPREV1 := REC_ARREDOND.QTCOMENST1;
            END IF;
            IF (REC_ARREDOND.QTCOMENST2 > 0) AND (MOD(REC_ARREDOND.QTCOMENST2, V_ARRED) > 0) THEN
               V_QTPREV2 := (FLOOR(REC_ARREDOND.QTCOMENST2/V_ARRED) + 1) * V_ARRED;
            ELSE
               V_QTPREV2 := REC_ARREDOND.QTCOMENST2;
            END IF;
            IF (REC_ARREDOND.QTCOMENST3 > 0) AND (MOD(REC_ARREDOND.QTCOMENST3, V_ARRED) > 0) THEN
               V_QTPREV3 := (FLOOR(REC_ARREDOND.QTCOMENST3/V_ARRED) + 1) * V_ARRED;
            ELSE
               V_QTPREV3 := REC_ARREDOND.QTCOMENST3;
            END IF;
            IF (REC_ARREDOND.QTCOMENST4 > 0) AND (MOD(REC_ARREDOND.QTCOMENST4,V_ARRED) > 0) THEN
               V_QTPREV4 := (FLOOR(REC_ARREDOND.QTCOMENST4/V_ARRED) + 1) * V_ARRED;
            ELSE
               V_QTPREV4 := REC_ARREDOND.QTCOMENST4;
            END IF;
            IF (V_QTPREV1 = 0) AND (V_QTPREV2 = 0) AND (V_QTPREV3 = 0) AND (V_QTPREV4 = 0) THEN
               IF (REC_ARREDOND.QTCOMENS > 0) AND (MOD(REC_ARREDOND.QTCOMENS, V_ARRED) > 0) THEN
                  V_QTPREV := (FLOOR(REC_ARREDOND.QTCOMENS/V_ARRED) + 1) * V_ARRED;
               ELSE
                  V_QTPREV := REC_ARREDOND.QTCOMENS;
               END IF;
            ELSE
               V_QTPREV := V_QTPREV1 + V_QTPREV2 + V_QTPREV3 + V_QTPREV4 ;
            END IF;
               UPDATE
                  CONSPLAN
               SET
                  QTCONSPLAN   = V_QTPREV,
                  QTCONSPLANT1 = V_QTPREV1,
                  QTCONSPLANT2 = V_QTPREV2,
                  QTCONSPLANT3 = V_QTPREV3,
                  QTCONSPLANT4 = V_QTPREV4,
                  QTCONSPLANSA = REC_ARREDOND.QTCOMENS,
                  QTCONPLAT1SA = REC_ARREDOND.QTCOMENST1,
                  QTCONPLAT2SA = REC_ARREDOND.QTCOMENST2,
                  QTCONPLAT3SA = REC_ARREDOND.QTCOMENST3,
                  QTCONPLAT4SA = REC_ARREDOND.QTCOMENST4
               WHERE
                  (CDFILICPLA   = REC_ARREDOND.CDFILIAL)     AND
                  (NRCONSPLAN   = REC_ARREDOND.NRREQUCONS)   AND
                  (CDSERVCPLA   = REC_ARREDOND.CDSERVFILI)   AND
                  (DTCONSPLAN   = REC_ARREDOND.DTDATA)       AND
                  (IDCONSPLAN   = REC_ARREDOND.IDTIPOCONS)   AND
                  (CDCLIECPLA   = REC_ARREDOND.CDCLIENTE)    AND
                  (CDDESTFICPLA = REC_ARREDOND.CDDESTFILI)   AND
                  (CDFAPRATCPLA = REC_ARREDOND.CDFILAUXPRAT) AND
                  (CDPRATCONSPL = REC_ARREDOND.CDPRATO)      AND
                  (CDPRODCPLA   = REC_ARREDOND.CDPRODUTO)    AND
                  (DTLIMITECPLA = REC_ARREDOND.DTLIMITE)     AND
                  (HRLIMITECPLA = REC_ARREDOND.HRLIMITE)     AND
                  (CDTPPRATCPLA = REC_ARREDOND.CDTIPOPRAT);
         END IF;
      END LOOP;
      COMMIT;
      CLOSE CUR_ARREDOND;
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('CONSUMO_PLANEJADO_PME','5.07.005'); END;
/
