/* Agent2b - Agendador de Eventos

Criao - 13/01/2005 - Adriano de Arajo

Essa procedure tem o objetivo de buscar a prxima data para execuo de um evento
de acordo com a sua parametrizao.

*/

CREATE OR REPLACE PROCEDURE
   CalcPrxExecEvento(P_CDGATILHO    IN GATILHO.CDGATILHO%TYPE,
                     P_DTPRXEXECGAT OUT GATILHO.DTPRXEXECGAT%TYPE) IS

  V_HRINIGATILHO GATILHO.HRINIGATILHO%TYPE;
  V_IDPERGATILHO GATILHO.IDPERGATILHO%TYPE;
  V_DTPROXEXEC   DATE;
  V_DTINIGATILHO DATE;

BEGIN

   P_DTPRXEXECGAT := NULL;

   SELECT
     HRINIGATILHO,
     IDPERGATILHO INTO V_HRINIGATILHO,
                       V_IDPERGATILHO

   FROM
     GATILHO
   WHERE
     CDGATILHO = P_CDGATILHO;

   IF V_IDPERGATILHO = '1' THEN  /* DIARIAMENTE */

      /*
      Criao - 10/01/2005 - Adriano de Arajo
      Sistema AEV - Agendador de Eventos

      Essa sentena sql tem o objetivo de buscar a prxima data para execuo de um evento
      de acordo com a sua parametrizao diria.  somado o intervalo de execuo ao campo
      de prxima de execuo do evento, quando esse campo est nulo a data inicial  a
      data de prxima execuo se no for menor que a data/hora atual, caso contrrio a data
      atual  a data de prxima execuo

      */

      SELECT
        MIN( NVL( GATILHO.DTPRXEXECGAT + TO_NUMBER( NVL(GATILHO.NRINTGATILHO, 1) ) , DATA.DTDATA) ) INTO V_DTPROXEXEC
      FROM
        GATILHO,
        DATA
      WHERE
        GATILHO.CDGATILHO = P_CDGATILHO AND

        TO_DATE( TO_CHAR(DATA.DTDATA, 'DD/MM/YYYY') || ' ' || HRINIGATILHO, 'DD/MM/YYYY HH24:MI:SS' ) >= SYSDATE AND

        DATA.DTDATA BETWEEN GATILHO.DTINIGATILHO AND GATILHO.DTFINGATILHO;


      IF V_DTPROXEXEC IS NOT NULL THEN

         UPDATE GATILHO
         SET IDGATILHOATI = 'A',
             DTPRXEXECGAT = TO_DATE( TO_CHAR(V_DTPROXEXEC, 'DD/MM/YYYY') || ' ' || V_HRINIGATILHO, 'DD/MM/YYYY HH24:MI:SS' )
         WHERE CDGATILHO = P_CDGATILHO;

      ELSE

         UPDATE GATILHO
         SET IDGATILHOATI = 'I',
             DTPRXEXECGAT = V_DTPROXEXEC
         WHERE CDGATILHO = P_CDGATILHO;

      END IF;

      COMMIT;

   END IF;

   IF V_IDPERGATILHO = '2' THEN       /* SEMANALMENTE */

      /*
      Criao - 08/01/2005 - Adriano de Arajo
      Sistema AEV - Agendador de Eventos

      Essa sentena sql tem o objetivo de buscar a prxima data para execuo de um evento
      de acordo com a sua parametrizao semanal.
      Primeiramente o sql traz todas as datas que esto entre a data atual de execuo + 1 e
      a data final do evento, isso  feito atravs do between com o campo dtdata. Dessas datas
      so mantidas apenas as que tem o dia da semana compatveis com os parametrizados, isso 
      sabido atravs o campo IDDIASEMGATI. Para cada dia retornado  dado um status para indicar
      em que semana est, partindo da primeira data os sete primeiros dias esto na semana 1,
      os sete dias seguintes esto na semana 2 e assim sucessivamente, isso  feito atravs da
      diviso do intervalo de dias por 7. Assim, como cada dia tem a indicao de que semana est,
      so considerados apenas os datas que esto nas semanas parametrizadas, pois o usurio pode
      indicar se o gatilho ser disparado a cada X semanas. As semanas parametrizadas so obtidas
      atravs do subselect no qual  dado um comando IN. Esse select funciona como uma srie do tipo
      1 + ( R * (N - 1) ), onde R indica a cada quantas semanas o evento deve ser executado e n comea
      de 1, no caso N=RowNum.
      */

      SELECT
        MIN(DATA.DTDATA) INTO V_DTPROXEXEC
      FROM
        GATILHO,
        DATA
      WHERE
        GATILHO.CDGATILHO = P_CDGATILHO AND

        TO_DATE( TO_CHAR(DATA.DTDATA, 'DD/MM/YYYY') || ' ' || HRINIGATILHO, 'DD/MM/YYYY HH24:MI:SS' ) >= SYSDATE AND

        DATA.DTDATA BETWEEN NVL(GATILHO.DTPRXEXECGAT + 1, GATILHO.DTINIGATILHO) AND GATILHO.DTFINGATILHO AND

        TO_CHAR(DATA.DTDATA, 'D') IN
          (DECODE(SUBSTR(GATILHO.IDDIASEMGATI, 1, 1), '1', 1,  0),  /* DOM */
           DECODE(SUBSTR(GATILHO.IDDIASEMGATI, 2, 1), '1', 2,  0),  /* SEG */
           DECODE(SUBSTR(GATILHO.IDDIASEMGATI, 3, 1), '1', 3,  0),  /* TER */
           DECODE(SUBSTR(GATILHO.IDDIASEMGATI, 4, 1), '1', 4,  0),  /* QUA */
           DECODE(SUBSTR(GATILHO.IDDIASEMGATI, 5, 1), '1', 5,  0),  /* QUI */
           DECODE(SUBSTR(GATILHO.IDDIASEMGATI, 6, 1), '1', 6,  0),  /* SEX */
           DECODE(SUBSTR(GATILHO.IDDIASEMGATI, 7, 1), '1', 7,  0) ) /* SAB */ AND

      ( ( TRUNC ( ( DTDATA - NVL(TRUNC(GATILHO.DTPRXEXECGAT) + 1, GATILHO.DTINIGATILHO) ) / 7 ) ) + 1 ) IN

       (SELECT
          1 + (TO_NUMBER(GATILHO.NRINTGATILHO) * (ROWNUM - 1)) AS SERIE_MULTIPLOS
        FROM
          DATA
        WHERE
          DTDATA >= TRUNC(SYSDATE) AND

          DTDATA BETWEEN NVL(GATILHO.DTPRXEXECGAT + 1, GATILHO.DTINIGATILHO) AND GATILHO.DTFINGATILHO);

      IF V_DTPROXEXEC IS NOT NULL THEN

         UPDATE GATILHO
         SET IDGATILHOATI = 'A',
             DTPRXEXECGAT = TO_DATE( TO_CHAR(V_DTPROXEXEC, 'DD/MM/YYYY') || ' ' || V_HRINIGATILHO, 'DD/MM/YYYY HH24:MI:SS' )
         WHERE CDGATILHO = P_CDGATILHO;

      ELSE

         UPDATE GATILHO
         SET IDGATILHOATI = 'I',
             DTPRXEXECGAT = V_DTPROXEXEC
         WHERE CDGATILHO = P_CDGATILHO;

      END IF;

      COMMIT;

   END IF;

   IF (V_IDPERGATILHO = '3') OR (V_IDPERGATILHO = '4') THEN
                                        /* Mensal (dias do ms) */                                     /* Mensal (dia til) */

      /*
      Criao - 10/01/2005 - Adriano de Arajo
      Sistema AEV - Agendador de Eventos

      Essa sentena sql tem o objetivo de buscar a prxima data para execuo de um evento
      de acordo com a sua parametrizao mensal.
      Primeiramente o sql traz todas as datas que esto entre a data atual de execuo + 1 e
      a data final do evento, isso  feito atravs do between com o campo dtdata. Existe um
      campo para cada ms onde a 1 posio indica que o primeiro dia foi escolhido para execuo
      do evento, a 2 posio indica o status do 2 dia e assim sucessivamente. O subselect DIASANO,
      traz todos os dias de um ano bissexto. Assim para cada ms  testado a posio DIASANO.DIA,
      que varia de 1 a 31, fazendo com que seja lida todas as posies de cada campo de ms e retornando
      o dd/mm quando status da posio do campo  1, o que indica que o dia do ms correspondente
      est configurado para execuo do evento.

      */

      SELECT
        MIN( TO_DATE( TO_CHAR(DATA.DTDATA, 'DD/MM/YYYY') || ' ' || HRINIGATILHO , 'DD/MM/YYYY HH24:MI') )  INTO V_DTPROXEXEC
      FROM
        GATILHO,
        DATA,

        (SELECT
           TO_CHAR(DTDATA, 'DD') AS DIA,
           TO_CHAR(DTDATA, 'DD/MM') AS DIAMES
         FROM
           DATA
         WHERE
           DTDATA BETWEEN '01/01/2004' AND '31/12/2004') DIASANO

      WHERE
        GATILHO.CDGATILHO = P_CDGATILHO  AND

        TO_DATE( TO_CHAR(DATA.DTDATA, 'DD/MM/YYYY') || ' ' || HRINIGATILHO, 'DD/MM/YYYY HH24:MI:SS' ) >= SYSDATE AND

        DATA.DTDATA BETWEEN NVL(TRUNC(GATILHO.DTPRXEXECGAT)+1, GATILHO.DTINIGATILHO) AND GATILHO.DTFINGATILHO AND

        NOT EXISTS
         ( SELECT
             FERIFILI.CDFILIAL
           FROM
             FERIFILI
           WHERE
             FERIFILI.CDFILIAL = DECODE(IDPERGATILHO, '4', CDFILIAL, '') AND
             FERIFILI.DTFERIFILI = DATA.DTDATA ) AND

        TO_CHAR(DTDATA, 'DD/MM') = DIASANO.DIAMES AND

        (
          DECODE( SUBSTR(GATILHO.IDDIAMESJANG, TO_NUMBER(DIASANO.DIA), 1), '1', DIASANO.DIA || '/01',  0) = DIASANO.DIAMES
        OR
          DECODE( SUBSTR(GATILHO.IDDIAMESFEVG, TO_NUMBER(DIASANO.DIA), 1), '1', DIASANO.DIA || '/02',  0) = DIASANO.DIAMES
        OR
          DECODE( SUBSTR(GATILHO.IDDIAMESFEVG, TO_NUMBER(DIASANO.DIA), 1), '1', DIASANO.DIA || '/03',  0) = DIASANO.DIAMES
        OR
          DECODE( SUBSTR(GATILHO.IDDIAMESFEVG, TO_NUMBER(DIASANO.DIA), 1), '1', DIASANO.DIA || '/04',  0) = DIASANO.DIAMES
        OR
          DECODE( SUBSTR(GATILHO.IDDIAMESFEVG, TO_NUMBER(DIASANO.DIA), 1), '1', DIASANO.DIA || '/05',  0) = DIASANO.DIAMES
        OR
          DECODE( SUBSTR(GATILHO.IDDIAMESFEVG, TO_NUMBER(DIASANO.DIA), 1), '1', DIASANO.DIA || '/06',  0) = DIASANO.DIAMES
        OR
          DECODE( SUBSTR(GATILHO.IDDIAMESFEVG, TO_NUMBER(DIASANO.DIA), 1), '1', DIASANO.DIA || '/07',  0) = DIASANO.DIAMES
        OR
          DECODE( SUBSTR(GATILHO.IDDIAMESFEVG, TO_NUMBER(DIASANO.DIA), 1), '1', DIASANO.DIA || '/08',  0) = DIASANO.DIAMES
        OR
          DECODE( SUBSTR(GATILHO.IDDIAMESFEVG, TO_NUMBER(DIASANO.DIA), 1), '1', DIASANO.DIA || '/09',  0) = DIASANO.DIAMES
        OR
          DECODE( SUBSTR(GATILHO.IDDIAMESFEVG, TO_NUMBER(DIASANO.DIA), 1), '1', DIASANO.DIA || '/10',  0) = DIASANO.DIAMES
        OR
          DECODE( SUBSTR(GATILHO.IDDIAMESFEVG, TO_NUMBER(DIASANO.DIA), 1), '1', DIASANO.DIA || '/11',  0) = DIASANO.DIAMES
        OR
          DECODE( SUBSTR(GATILHO.IDDIAMESFEVG, TO_NUMBER(DIASANO.DIA), 1), '1', DIASANO.DIA || '/12',  0) = DIASANO.DIAMES
        );

      IF V_DTPROXEXEC IS NOT NULL THEN

         UPDATE GATILHO
         SET IDGATILHOATI = 'A',
             DTPRXEXECGAT = TO_DATE( TO_CHAR(V_DTPROXEXEC, 'DD/MM/YYYY') || ' ' || V_HRINIGATILHO, 'DD/MM/YYYY HH24:MI:SS' )
         WHERE CDGATILHO = P_CDGATILHO;

      ELSE

         UPDATE GATILHO
         SET IDGATILHOATI = 'I',
             DTPRXEXECGAT = V_DTPROXEXEC
         WHERE CDGATILHO = P_CDGATILHO;

      END IF;

      COMMIT;

   END IF;

   IF V_IDPERGATILHO = '5' THEN   /* Semana/Ms */

      /*
      Criao - 11/01/2005 - Adriano de Arajo
      Sistema AEV - Agendador de Eventos

      Essa sentena sql tem o objetivo de buscar a prxima data para execuo de um evento
      de acordo com a sua parametrizao Semanal/Mensal.
      Primeiramente o sql traz todas as datas que esto entre a data atual de execuo + 1 e
      a data final do evento, isso  feito atravs do between com o campo dtdata. Existem campos
      que indicam para cada dia da semana em qual ocorrncia do ms o evento deve ser executado.
      O subselect nomeado DIASOCORR, traz todos os dias dos anos do perodo em que o evento deve
      ser executado, estando os sete primeiros dias na ocorrncia 1, os prximos 7 dias na ocorrncia 2,
      e assim sucessivamente para cada ms(o primeiro dia de cada ms est na ocorrncia 1). Os dias que
      estiverem nas ocorrncias parametrizadas para cada dia da semana  que sero trazidas.

      */

      SELECT
        MIN(DTDATA)  INTO V_DTPROXEXEC
      FROM
        GATILHO,

        (SELECT
           TO_NUMBER( TO_CHAR(DTDATA, 'DD') ) POS
         FROM
           DATA
         WHERE
           DTDATA BETWEEN '01/01/2005' AND '05/01/2005') POSOCORR,

        (SELECT
           TO_CHAR(DTDATA, 'DD') POS
         FROM
           DATA
         WHERE
           DTDATA BETWEEN '01/01/2005' AND '12/01/2005') POSMES,

        (SELECT
           DTDATA,
           ( ( TRUNC ( ( DTDATA - TO_DATE( '01/'||TO_CHAR(DTDATA, 'MM/YYYY') ) ) / 7 ) ) + 1 ) AS OCORR

         FROM
           GATILHO,
           DATA
          WHERE
           GATILHO.CDGATILHO = P_CDGATILHO AND
           DATA.DTDATA BETWEEN '01/01/'||TO_CHAR(DTINIGATILHO, 'YYYY') AND '31/12/'||TO_CHAR(DTFINGATILHO, 'YYYY') ) DIASOCORR

      WHERE
        GATILHO.CDGATILHO = P_CDGATILHO AND

        TO_DATE( TO_CHAR(DIASOCORR.DTDATA, 'DD/MM/YYYY') || ' ' || HRINIGATILHO, 'DD/MM/YYYY HH24:MI:SS' ) >= SYSDATE AND

        DIASOCORR.DTDATA BETWEEN NVL(TRUNC(GATILHO.DTPRXEXECGAT) + 1, GATILHO.DTINIGATILHO) AND GATILHO.DTFINGATILHO AND

        TO_CHAR(DIASOCORR.DTDATA, 'MM') = DECODE( SUBSTR(GATILHO.IDMESGATILHO, TO_NUMBER(POSMES.POS), 1), '1', POSMES.POS, 0 ) AND

        TO_CHAR(DIASOCORR.DTDATA, 'D') || DIASOCORR.OCORR  IN

        ( '1' || DECODE( SUBSTR(GATILHO.IDOCODOMGATI, POSOCORR.POS, 1), '1', POSOCORR.POS,  0 ),  /* OCORRNCIAS DE DOM */
          '2' || DECODE( SUBSTR(GATILHO.IDOCOSEGGATI, POSOCORR.POS, 1), '1', POSOCORR.POS,  0 ),  /* OCORRNCIAS DE SEG */
          '3' || DECODE( SUBSTR(GATILHO.IDOCOTERGATI, POSOCORR.POS, 1), '1', POSOCORR.POS,  0 ),  /* OCORRNCIAS DE TER */
          '4' || DECODE( SUBSTR(GATILHO.IDOCOQUAGATI, POSOCORR.POS, 1), '1', POSOCORR.POS,  0 ),  /* OCORRNCIAS DE QUA */
          '5' || DECODE( SUBSTR(GATILHO.IDOCOQUIGATI, POSOCORR.POS, 1), '1', POSOCORR.POS,  0 ),  /* OCORRNCIAS DE QUI */
          '6' || DECODE( SUBSTR(GATILHO.IDOCOSEXGATI, POSOCORR.POS, 1), '1', POSOCORR.POS,  0 ),  /* OCORRNCIAS DE SEX */
          '7' || DECODE( SUBSTR(GATILHO.IDOCOSABGATI, POSOCORR.POS, 1), '1', POSOCORR.POS,  0 ) );/* OCORRNCIAS DE SAB */

      IF V_DTPROXEXEC IS NOT NULL THEN

         UPDATE GATILHO
         SET IDGATILHOATI = 'A',
             DTPRXEXECGAT = TO_DATE( TO_CHAR(V_DTPROXEXEC, 'DD/MM/YYYY') || ' ' || V_HRINIGATILHO, 'DD/MM/YYYY HH24:MI:SS' )
         WHERE CDGATILHO = P_CDGATILHO;

      ELSE

         UPDATE GATILHO
         SET IDGATILHOATI = 'I',
             DTPRXEXECGAT = V_DTPROXEXEC
         WHERE CDGATILHO = P_CDGATILHO;

      END IF;

      COMMIT;

   END IF;

   IF V_IDPERGATILHO = '6' THEN       /* SEMANALMENTE */

      SELECT
        MIN(DTPRXEXECGAT),
        MIN(DTINIGATILHO) INTO V_DTPROXEXEC,
                               V_DTINIGATILHO
      FROM
        GATILHO
      WHERE
        CDGATILHO = P_CDGATILHO;

      IF V_DTPROXEXEC IS NULL THEN

         V_DTPROXEXEC := TO_DATE( TO_CHAR(V_DTINIGATILHO, 'DD/MM/YYYY') || ' ' || V_HRINIGATILHO, 'DD/MM/YYYY HH24:MI:SS' );

         UPDATE GATILHO
         SET IDGATILHOATI = 'A',
             DTPRXEXECGAT = V_DTPROXEXEC
         WHERE CDGATILHO = P_CDGATILHO;

      ELSE

         V_DTPROXEXEC := NULL;

         UPDATE GATILHO
         SET IDGATILHOATI = 'I'
         WHERE CDGATILHO = P_CDGATILHO;

      END IF;

      COMMIT;

   END IF;

   IF V_DTPROXEXEC IS NOT NULL THEN

     P_DTPRXEXECGAT := TO_DATE( TO_CHAR(V_DTPROXEXEC, 'DD/MM/YYYY') || ' ' || V_HRINIGATILHO, 'DD/MM/YYYY HH24:MI:SS' );

   END IF;

END;
/
